package com.example.springbootdownloadzip.controller;

import org.apache.logging.log4j.util.Base64Util;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;


@RestController
@RequestMapping("/test")
public class ZipController {


    // http://localhost:8188/test/download/package3
    @GetMapping("/download/package3")
    public void compressedDownload3(HttpServletResponse response) {
        /*
        第二次优化，封装工具类，封装异常
        需要封装两个工具类

         */

    }

    // http://localhost:8188/test/download/package2
    @GetMapping("/download/package2")
    public void compressedDownload2(HttpServletResponse response) throws IOException {
        //完成逻辑拆分
        /*

        第一步 确定 写入的文件，注意的是，文件可能是逻辑内生成的，也可能是只有地址的网络图片


            第二步 放到压缩文件中

           1 文件放入压缩文件
           2 下载压缩文件

           对于文件放入压缩文件，需要的内容为
                1 存放在压缩文件的路径 比如 /image/2022/xxxxxx.png
                2 输入流  InputStream 类型

           经过上面的步骤，即可得到一个zip的文件流

           第三步 为 下载zip文件 需要参数为 下载文件的文件名
         */

        //zip文件名
        SimpleDateFormat formatter = new SimpleDateFormat("优化测试yyyy年MM月dd日HH时mm分ss秒");
        String zipFileName = formatter.format(new Date()) + ".zip";

        //模拟参数进入
        HashMap<String, InputStream> zipContent = new HashMap<>(2);

        String file1 = "https://cdn.coderfix.cn/202202/image-20220225153233962.png";
        String file2 = "https://cdn.coderfix.cn/202202/image-20220225153414861.png";

        zipContent.put("/测试路径/测试命名.png", getImg(file1));
        zipContent.put("/测试路径2/测试路径/测试命名2.png", getImg(file2));

        //处理zip
        String directory = "/tmp/";
        String strZipPath = directory + "\\" + zipFileName;
        ZipOutputStream zipStream = null;
        FileInputStream zipSource = null;
        BufferedInputStream bufferStream = null;
        File zipFile = new File(strZipPath);
        try {
            //构造最终压缩包的输出流
            zipStream = new ZipOutputStream(new FileOutputStream(zipFile));
            for (Map.Entry<String, InputStream> entry : zipContent.entrySet()) {
                ZipEntry zipEntry = new ZipEntry(entry.getKey());//在压缩目录中文件的名字
                zipStream.putNextEntry(zipEntry);//定位该压缩条目位置，开始写入文件到压缩包中
                bufferStream = new BufferedInputStream(entry.getValue(), 1024 * 10);
                int read = 0;
                byte[] buf = new byte[1024 * 10];
                while ((read = bufferStream.read(buf, 0, 1024 * 10)) != -1) {
                    zipStream.write(buf, 0, read);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (null != bufferStream) bufferStream.close();
                if (null != zipStream) {
                    zipStream.flush();
                    zipStream.close();
                }
                if (null != zipSource) {
                    zipSource.close();
                }
            } catch (IOException exception) {
                exception.printStackTrace();
            }
        }

        if (zipFile.exists()) {
            downImg(response, zipFileName, strZipPath);
            zipFile.delete();
        }
    }

    private InputStream getImg(String fileUrl) throws IOException {
        //需要封装异常处理
        URL url = new URL(fileUrl);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setConnectTimeout(3 * 1000);
        conn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");
        return conn.getInputStream();
    }


    //文件打包下载测试 http://localhost:8188/test/download/package
    @GetMapping("/download/package")
    public void compressedDownload(HttpServletResponse response) {
        //伪造文件名数组
        String[] names = {"10.png", "11.png"};
        //伪造文件所在路径数组
        String[] paths = {"https://cdn.coderfix.cn/202202/image-20220225153233962.png",
                "https://cdn.coderfix.cn/202202/image-20220225153414861.png"};

        //创建临时目录以存放生成的压缩包文件
        String directory = "/tmp/";
        File directoryFile = new File(directory);

        //设置最终输出zip文件的目录+文件名
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy年MM月dd日HH时mm分ss秒");
        String zipFileName = formatter.format(new Date()) + ".zip";
        String strZipPath = directory + "\\" + zipFileName;

        //创建流
        ZipOutputStream zipStream = null;
        FileInputStream zipSource = null;
        BufferedInputStream bufferStream = null;
        File zipFile = new File(strZipPath);
        try {
            //构造最终压缩包的输出流
            zipStream = new ZipOutputStream(new FileOutputStream(zipFile));
            for (int i = 0; i < paths.length; i++) {
                //解码获取真实路径与文件名
                String realFileName = java.net.URLDecoder.decode(names[i], "UTF-8");
                String realFilePath = java.net.URLDecoder.decode(paths[i], "UTF-8");
//                File file = new File(realFilePath);
                //根据url获取输入流
                URL url = new URL(realFilePath);
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
//                设置超时间为3秒
                conn.setConnectTimeout(3 * 1000);
//                防止屏蔽程序抓取而返回403错误
                conn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");

//                得到输入流
                InputStream inputStream = conn.getInputStream();


                if (inputStream.available() > 0) {
                    ZipEntry zipEntry;
                    if (i > 0) {
                        zipEntry = new ZipEntry("/aaa/" + realFileName);//在压缩目录中文件的名字
                    } else {
                        zipEntry = new ZipEntry("/bbb/vvv/" + realFileName);//在压缩目录中文件的名字
                    }

                    zipStream.putNextEntry(zipEntry);//定位该压缩条目位置，开始写入文件到压缩包中
                    bufferStream = new BufferedInputStream(inputStream, 1024 * 10);
                    int read = 0;
                    byte[] buf = new byte[1024 * 10];
                    while ((read = bufferStream.read(buf, 0, 1024 * 10)) != -1) {
                        zipStream.write(buf, 0, read);
                    }
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            //关闭流
            try {
                if (null != bufferStream) bufferStream.close();
                if (null != zipStream) {
                    zipStream.flush();
                    zipStream.close();
                }
                if (null != zipSource) {
                    zipSource.close();
                }
            } catch (IOException exception) {
                exception.printStackTrace();
            }
        }

        if (zipFile.exists()) {
            downImg(response, zipFileName, strZipPath);
            zipFile.delete();
        }
    }

    public void downImg(HttpServletResponse response, String filename, String path) {
        if (filename != null) {
            FileInputStream is = null;
            BufferedInputStream bs = null;
            OutputStream os = null;
            try {
                File file = new File(path);
                if (file.exists()) {
                    //设置Headers
                    response.setHeader("Content-Type", "application/octet-stream");
                    //设置下载的文件的名称-该方式已解决中文乱码问题
                    response.setHeader("Content-Disposition", "attachment;filename=" + new String(filename.getBytes("gb2312"), "ISO8859-1"));
                    is = new FileInputStream(file);
                    bs = new BufferedInputStream(is);
                    os = response.getOutputStream();
                    byte[] buffer = new byte[1024];
                    int len = 0;
                    while ((len = bs.read(buffer)) != -1) {
                        os.write(buffer, 0, len);
                    }
                } else {
                    String error = Base64Util.encode("下载的文件资源不存在");
                    response.sendRedirect("/imgUpload/imgList?error=" + error);
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            } finally {
                try {
                    if (is != null) {
                        is.close();
                    }
                    if (bs != null) {
                        bs.close();
                    }
                    if (os != null) {
                        os.flush();
                        os.close();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

}
